"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Result {
    constructor(status, value, cookies, headers, redirectUrl, local) {
        this.status = status;
        this.value = value;
        this.cookies = cookies;
        this.headers = headers;
        this.redirectUrl = redirectUrl;
        this.local = local;
    }
}
exports.Result = Result;
function ensureNoValueOnOptions(options) {
    if (!options) {
        return {};
    }
    const optionsWithValue = options;
    // Make sure no one is doing any funny business with types
    // Technically the ResultOptionsWithValue will cast to ResultOptionsNoValue
    // given all properties are optional. So this is explicitly setting value to undefined
    // in case it was set.
    optionsWithValue.value = undefined;
    return optionsWithValue;
}
class DataResult extends Result {
    constructor(status, redirectUrl, options) {
        super(status, options === null || options === void 0 ? void 0 : options.value, options === null || options === void 0 ? void 0 : options.cookies, options === null || options === void 0 ? void 0 : options.headers, redirectUrl, options === null || options === void 0 ? void 0 : options.local);
    }
    static ok(options) {
        const status = (options === null || options === void 0 ? void 0 : options.value) ? 200 : 204;
        return new DataResult(status, undefined, options);
    }
    static error(value, options) {
        const optionsWithValue = options !== null && options !== void 0 ? options : {};
        optionsWithValue.value = value;
        return new DataResult(400, undefined, optionsWithValue);
    }
    static redirect(url, status = 302, options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new DataResult(status, url, sanitized);
    }
    static notFound(options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new DataResult(404, undefined, sanitized);
    }
    static unauthorized(options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new DataResult(401, undefined, sanitized);
    }
}
exports.DataResult = DataResult;
class ViewResult extends Result {
    constructor(status, template, redirectUrl, options) {
        super(status, options === null || options === void 0 ? void 0 : options.value, options === null || options === void 0 ? void 0 : options.cookies, options === null || options === void 0 ? void 0 : options.headers, redirectUrl, options === null || options === void 0 ? void 0 : options.local);
        this.template = template;
    }
    static ok(template, options) {
        return new ViewResult(200, template, undefined, options);
    }
    static redirect(url, status = 302, options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new ViewResult(status, undefined, url, sanitized);
    }
    static error(template, options) {
        return new ViewResult(400, template, undefined, options);
    }
    static notFound(template, options) {
        return new ViewResult(404, template, undefined, options);
    }
    static unauthorized(template, options) {
        return new ViewResult(401, template, undefined, options);
    }
}
exports.ViewResult = ViewResult;
class MiddlewareResult extends Result {
    constructor(status, cont, end, redirectUrl, options) {
        super(status, options === null || options === void 0 ? void 0 : options.value, options === null || options === void 0 ? void 0 : options.cookies, options === null || options === void 0 ? void 0 : options.headers, redirectUrl, options === null || options === void 0 ? void 0 : options.local);
        this.continue = cont;
        this.end = end;
    }
    /**
     * Advances to the next middleware. To pass data along to the next middleware handler or route handler
     * set options.value. Context.local will be set to the merged values of previous Middleware.continue.
     * @param options
     */
    static continue(options) {
        // Status being set for compatibility with "result" this may shift as we flesh out
        // middleware a bit more.
        return new MiddlewareResult(200, true, false, undefined, options);
    }
    /**
     * Will end the request with the given status and setting the body to options.value. For view routes the render
     * function will be skipped so any rendering of templates would need to be done explicitly in the middleware.
     *
     * This choice is intentional to allow middleware to handle things like caching or other events that may require it to
     * end the request with some content.
     *
     * @param status
     * @param options
     */
    static end(status, options) {
        return new MiddlewareResult(status, false, true, undefined, options);
    }
    static error(value, options) {
        const optionsWithValue = options !== null && options !== void 0 ? options : {};
        optionsWithValue.value = value;
        return new MiddlewareResult(400, false, false, undefined, optionsWithValue);
    }
    static redirect(url, status = 302, options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new MiddlewareResult(status, false, false, url, sanitized);
    }
    static notFound(options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new MiddlewareResult(404, false, false, undefined, sanitized);
    }
    static unauthorized(options) {
        const sanitized = ensureNoValueOnOptions(options);
        return new MiddlewareResult(401, false, false, undefined, sanitized);
    }
}
exports.MiddlewareResult = MiddlewareResult;
//# sourceMappingURL=result.js.map